// SerialPort.cpp: implementation of the CSerialPort class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "EtchASketch.h"
#include "SerialPort.h"
#include "PortException.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CSerialPort::CSerialPort()
{
	h = CreateFile("COM1",
		GENERIC_READ|GENERIC_WRITE,
		0,NULL,
		OPEN_EXISTING,FILE_FLAG_OVERLAPPED,NULL);
	
    if(h == INVALID_HANDLE_VALUE) {
		h = CreateFile("COM2",
			GENERIC_READ|GENERIC_WRITE,
			0,NULL,
			OPEN_EXISTING,FILE_FLAG_OVERLAPPED,NULL);
	}
	
	if(h == INVALID_HANDLE_VALUE)
		throw new PortException("Could not get Port Handle");
	
	// set timeouts
	COMMTIMEOUTS cto = { 2, 1, 1, 0, 0 };
	
	if(!SetCommTimeouts(h,&cto))
		throw new PortException("SetCommTimeouts failed");
	
	// set DCB
	memset(&dcb,0,sizeof(dcb));
	dcb.DCBlength = sizeof(dcb);
	
	//set the Default Values for the port
	dcb.BaudRate = 9600;
	dcb.fBinary = 1;
	dcb.fDtrControl = DTR_CONTROL_DISABLE;//ENABLE;
	
	dcb.fOutxCtsFlow = 0;
	dcb.fRtsControl = DTR_CONTROL_DISABLE; //DTR_CONTROL_HANDSHAKE;
	
	dcb.Parity = NOPARITY;
	dcb.StopBits = ONESTOPBIT;
	dcb.ByteSize = 8;
	
	if(!SetCommState(h,&dcb))
		throw new PortException("SetCommState failed");
	
	//Set the virtual allocation
	
	//Initialize the threads
	inThread = AfxBeginThread(PortInThread,this);
	if( inThread == INVALID_HANDLE_VALUE ) {
		throw new PortException("CreateThread failed");
		return;
	}
	
	outThread = AfxBeginThread(PortOutThread,this);
	if( outThread == INVALID_HANDLE_VALUE ) {
		throw new PortException("CreateThread failed");
		return;
	}
	
}

CSerialPort::~CSerialPort()
{
	
	//Kill the in and out threads
	inThread->PostThreadMessage(WM_DESTROY,0L,0L);
	outThread->PostThreadMessage(WM_DESTROY,0L,0L);
	
	SendToPort(NULL,0);
	ReadFromPort(NULL,0);
	
	CloseHandle(h);
}

BOOL CSerialPort::SendToPort(byte* bArray, int iSize)
{
	try {
		int	count;
		bSend = bArray;
		iSizeOut = iSize;
		
		count = outThread->ResumeThread();
		
		//Make sure that the thread has stopped before resuming
		while(count == 0)
			count = outThread->ResumeThread();
		
		return TRUE; 
	}
	catch(...) {
		return FALSE;
	}
}

BOOL CSerialPort::ReadFromPort(byte* bArray, int iSize)
{
	try {
		int	count;
		bRead = bArray;
		iSizeIn = iSize;
		
		count = inThread->ResumeThread();
		
		//Make sure that the thread has stopped before resuming
		//But unlike the above thread, I must stop...
		while(count <= 0) {
			count = outThread->ResumeThread();
			if(count < 0) return FALSE;
		}
		
		return TRUE; 
	}
	catch(...) {
		return FALSE;
	}
}

UINT PortOutThread(LPVOID lpv) {
	CSerialPort *cspThis = (CSerialPort *)lpv;
	OVERLAPPED ov;
	unsigned long write;
	MSG msg;
	
	ZeroMemory(&ov,sizeof(ov));
	ov.hEvent = CreateEvent(NULL,FALSE,FALSE,NULL);
	if(ov.hEvent == INVALID_HANDLE_VALUE) {
		throw new PortException("CreateEvent failed");
		SetCommMask(cspThis->h,0);
		return 0;
	}
	
	//Wait until called
	cspThis->outThread->SuspendThread();
	
	//Look for a destroy message
	if(::PeekMessage(&msg,NULL, 0, 0, PM_REMOVE) &&
		msg.message == WM_DESTROY) {
		cspThis->outThread = 0;
		return 0;
	}
	
	// send it to the com port
	while(1) {
		
		if( !WriteFile(cspThis->h,cspThis->bSend,cspThis->iSizeOut,&write,&ov) ) {
			if(GetLastError() == ERROR_IO_PENDING) {
				if( !GetOverlappedResult(cspThis->h,&ov,&write,TRUE) ) 
					throw new PortException("GetOverlappedResult failed");
			}
			else throw new PortException("WriteFile failed");
		}
		cspThis->outThread->SuspendThread(); //Done, reset the value
		
		if(::PeekMessage(&msg,NULL, 0, 0, PM_REMOVE) &&
			msg.message == WM_DESTROY) {
			return 0;
		}
	}
}

UINT PortInThread(LPVOID lpv) {
	CSerialPort *cspThis = (CSerialPort *)lpv;
	OVERLAPPED ov;
	unsigned long write;
	MSG msg;
	
	ZeroMemory(&ov,sizeof(ov));
	ov.hEvent = CreateEvent(NULL,FALSE,FALSE,NULL);
	if(ov.hEvent == INVALID_HANDLE_VALUE) {
		throw new PortException("CreateEvent failed");
		SetCommMask(cspThis->h,0);
		return 0;
	}
	
	//Wait until called
	cspThis->inThread->SuspendThread();
	
	//Look for a destroy message
	if(::PeekMessage(&msg,NULL, 0, 0, PM_REMOVE) &&
		msg.message == WM_DESTROY)	{
		return 0;
	}
	
	// recieve from the com port
	while(1) {
		
		if( !ReadFile(cspThis->h,cspThis->bRead,cspThis->iSizeIn,&write,&ov) ) {
			if(GetLastError() == ERROR_IO_PENDING) {
				if( !GetOverlappedResult(cspThis->h,&ov,&write,TRUE) ) 
					throw new PortException("GetOverlappedResult failed");
			}
			else throw new PortException("ReadFile failed");
        }
		else cspThis->inThread->SuspendThread(); //Done, reset the value
		
		//Look for a destroy message
		if(::PeekMessage(&msg,NULL, 0, 0, PM_REMOVE) &&
			msg.message == WM_DESTROY)	{	
			return 0;
		}
	}
	
	return 0;
}
